 (* JES Inc. presents:  MTR/TOP 5.1 (1993) *)

 (*$V-,S-,R-,I-*)
 IMPLEMENTATION MODULE Manager;
IMPORT IO;
(* FROM IO      IMPORT WrStr, WrCard, WrLn, RdChar, KeyPressed, RdCharDirect;
*)
 FROM Lib     IMPORT RANDOM, Terminate;
 FROM SYSTEM  IMPORT PROCESS,NEWPROCESS,TRANSFER,IOTRANSFER,EI,DI,
		     GetFlags,SetFlags,CurrentProcess;
 FROM Storage IMPORT ALLOCATE, DEALLOCATE;

TYPE

  Task = POINTER TO TaskDescriptor;

  TaskDescriptor = RECORD
		     next     : Task; (* ᫥騩  ।  *)
		     priority : CARDINAL;
		     cor      : PROCESS;
		     due      : CARDINAL;
		     nextdue  : Task;
		     MaxR,                 (* ᨬ ᮢ *)
                     ActR,                 (* ⨢ ᮢ *)
                     OstR,                 (* ⠫ ᮢ *)
                     NeedR,                (* 襭 ᮢ *)
                     NumProc : CARDINAL;   (*   *)
		     ListR    : RESOURCE;  (* ᯨ᮪ 뤥 ᮢ *)
		     Finish   : BOOLEAN;   (* 䫠  .  *)
		   END;

  VAR
    cp : Task;  (* ⥪ ⨢  + । ⨢  *)
    dq : Task;  (* । , 믮 delay *)
    wq : Task;  (* । , 믮 delay  稢
                    ࠭ ,    
                   ⠢  cp - ஢ *)
    RQ : Task;  (* । ᮢ,   *)

    SchedProc  : PROCESS;
    SchedStack : ADDRESS;
    Started    : BOOLEAN;

(*$W+*)  (* Volatile variables *)
VAR
  Stop       : BOOLEAN;
  LockNest   : CARDINAL;   (* No of nested locks *)
			   (* only safe to slice if zero *)
  SchedTime  : CARDINAL;
(*$W-*)

 VAR
     ResourceLim : CARDINAL;      (* ᥣ ᮢ *)
     ResourceSet : CARDINAL;      (*  ᮢ *)
     ResourceList : RESOURCE;     (* ᮪ ᢮ ᮢ *)

     WinStat : Window.WinType; (*  ﭨ ᮢ *)
     WinBank : Window.WinType; (*  ⥬ ᮮ饭 *)
     WinRes  : Window.WinType; (*  । ᮢ *)
     WinLoc  : Window.WinType; (*  饩 ⨪ *)
     GlNP,                     (* ⢮ ⨢ ᮢ *)
     GlNPZ : CARDINAL;         (* ⢮ ࠡ⠢ ᮢ *)

 (* *** 楤   PROCESS *** *)

PROCEDURE QInsert(T: Task; VAR Q: Task);
(* inserts task after last task in Q with greater or equal priority *)
VAR
  q,qb : Task;
BEGIN
  q  := Q;
  qb := NIL;
  WHILE (q<>NIL)AND(T^.priority<=q^.priority) DO
    qb := q;
    q := q^.next;
  END;
  IF qb=NIL THEN
    Q := T;
  ELSE
    qb^.next := T;
  END;
  T^.next := q;
END QInsert;

PROCEDURE AddReadyProcess(T: Task);
(* adds new process to ready list
   NB gets added ahead of current process if at same priority
*)
VAR
  mp    : PROCESS;
  oldcp : Task;
  ie    : CARDINAL;
BEGIN
  ie := GetFlags(); DI;
  IF cp=NIL THEN
    QInsert(T,cp);          (* add new process *)
  ELSE
    oldcp := cp;            (* remove current process *)
    cp := cp^.next;
    QInsert(T,cp);          (* add new process *)
    QInsert(oldcp,cp);      (* add current process *)
  END;
  mp := cp^.cor;
  TRANSFER (mp,mp);
  SetFlags(ie);
END AddReadyProcess;

(* *)
PROCEDURE StartProcess(P: PROC; N: CARDINAL; Pr: CARDINAL;
		       MaxRes : CARDINAL; nump : CARDINAL);
VAR
  wsp: ADDRESS;
  np : Task;
BEGIN
  ALLOCATE (wsp, N);
  ALLOCATE (np, SIZE(TaskDescriptor));
  WITH np^ DO
       priority := Pr;
       MaxR := MaxRes;
       OstR := MaxRes;
       ActR := 0;
       ListR := NIL;
       NeedR := 0;
       Finish := TRUE;
       NumProc := nump;
       next := NIL;
       nextdue := NIL;
       IF NumProc>0 THEN  INC(GlNP); END;
      END;
  NEWPROCESS (P, wsp, N , np^.cor);
  AddReadyProcess(np);
END StartProcess;


(* *)
PROCEDURE CheckTimeQ;
VAR
  ta,tb,tn : Task;
BEGIN
  ta := dq;
  tb := NIL;
  WHILE ta <> NIL DO
    tn := ta^.nextdue;
    IF ta^.due = SchedTime THEN
      IF tb = NIL THEN dq := tn ELSE tb^.nextdue := tn END;
      ta^.nextdue := wq;
      wq := ta;
    ELSE
      tb := ta;
    END;
    ta := tn;
  END;
END CheckTimeQ;
(* *)

PROCEDURE Slice;
(* Clears waiting queue *)
(* Then schedules next ready process if it is of equal priority *)
VAR
  nextt,oldt,ta : Task;
BEGIN
  IF LockNest = 0 THEN
    (* move waiting queue to the ready queue *)
    (* set up by CheckTimeQueue *)
    WHILE wq <> NIL DO
      ta := wq; wq := wq^.nextdue;
      ta^.next := NIL;
      ta^.nextdue := NIL;
      QInsert(ta,cp);
    END;
    (* now do slice *)
    nextt := cp^.next;
    IF (nextt <> NIL) AND (nextt^.priority = cp^.priority) THEN (* slice *)
      oldt := cp; cp := nextt;
      oldt^.next := NIL;
      QInsert(oldt,cp); (* insert old cp at end of processes *)
    END;
  END;
END Slice;


MODULE SS[1]; (* IRQ 1: timer interrupt *)

 IMPORT Stop,Task,PROCESS,IOTRANSFER,TRANSFER,SchedTime,cp,DI,
	Slice,CheckTimeQ;
 EXPORT Scheduler;

(* *)
PROCEDURE Scheduler;
VAR
  nextt,
  oldt  : Task;
  op,np : PROCESS;
  Int8  : PROC;

TYPE
  code  = ARRAY[0..2] OF SHORTCARD;

CONST
  Int8code = code(0CDH,08H,0CBH); (* INT 08H / RETF *)

BEGIN
  DI;
  Int8 := PROC(ADR(Int8code));
  Stop := FALSE;
  SchedTime := 0;
(*  LOOP *)
    np := cp^.cor;
    LOOP
      IOTRANSFER(op,np,8);
      Int8;
      INC(SchedTime);
      IF Stop THEN EXIT END;
      CheckTimeQ;
      Slice;
      np := cp^.cor;
    END;
    Stop := FALSE;
(*    TRANSFER(op,np);    no return until restarted *)
(*  END; *)
END Scheduler;


END SS;

(* *)
PROCEDURE Idler; (* always on cp chain *)
VAR
  i : CARDINAL;
BEGIN
  LOOP INC(i);
  END;
END Idler;

(* *)
PROCEDURE StartScheduler;
VAR
  ie : CARDINAL;
BEGIN
  ie := GetFlags(); DI;
  IF NOT Started THEN
    Started := TRUE;
    IF SchedStack = NIL THEN (* first time *)
      ALLOCATE( SchedStack, 512 );
      NEWPROCESS( Scheduler, SchedStack, 512, SchedProc );
    END;
    TRANSFER( cp^.cor, SchedProc );
  END;
  SetFlags(ie);
END StartScheduler;

(* *)
PROCEDURE StopScheduler;
VAR
  ie : CARDINAL;
BEGIN
  ie := GetFlags();
  IF Started THEN
    EI ;
    Started := FALSE;
    Stop := TRUE;
    WHILE Stop DO END;
  END;
  Message ('>System:Scheduler stop.');
  SetFlags(ie);
END StopScheduler;
(* *)
PROCEDURE Delay(T: CARDINAL);
(* Waits T time slices *)
(* 0 will swap to next process of equal priority, without delaying *)
VAR
  mp : PROCESS;
  ie : CARDINAL;
BEGIN
  ie := GetFlags(); DI;
  IF T>0 THEN
    cp^.due := SchedTime + T;
    cp^.nextdue := dq;
    dq := cp;
    cp := cp^.next;
    dq^.next := NIL;
  mp := cp^.cor;
  TRANSFER(mp,mp);
  END;
  SetFlags(ie);
END Delay;

(* *)
PROCEDURE Lock;
(* Critical region lock - prevents timeslicing *)
(* may be nested *)
BEGIN
  INC(LockNest);
END Lock;

(* *)
PROCEDURE Unlock;
(* Unlock procedure, always paired with a call to Lock.
   Will de-schedule current process if there are ready processes
   of equal priority *)

VAR
  ie : CARDINAL;
  mp : PROCESS;
BEGIN
  ie := GetFlags(); DI;
  IF LockNest <= 1
     THEN LockNest := 0;  mp := cp^.cor;
          Slice;
          IF mp <> cp^.cor THEN mp:=cp^.cor; TRANSFER(mp,mp); END;
  ELSE
    DEC(LockNest);
  END;
  SetFlags(ie);
END Unlock;



VAR
  Continue : PROC;
(* *)
PROCEDURE CloseDown;
BEGIN
  StopScheduler;
  Continue;
END CloseDown;

 (* *** 訫 ᠭ 楤   PROCESS  *** *)


 (* *** 楤, ॠ騥  । ᮢ, *** *)
 (* ***  㯨  ᯮ짮 ⬠   *** *)

 (* ***  ⬠   *** *)

 PROCEDURE NextTask ( VAR f : CARDINAL; VAR tsk : Task) : BOOLEAN;
 (* ⢫ ᫥⥫  ᬮ ᯨ᪠ ,
    ᮤঠ  । cp,RQ,wq,dq *)
     BEGIN
	   CASE f OF
		   | 0 :  tsk:=cp; f:=1;
		   | 1 :  IF (tsk^.next<>NIL)
			     THEN tsk:=tsk^.next;
			     ELSE IF (RQ<>NIL) THEN tsk:=RQ; f:=2;
				  ELSE IF (wq<>NIL) THEN tsk:=wq; f:=4;
				       ELSE IF (dq<>NIL) THEN tsk:=dq; f:=3;
					    ELSE RETURN (FALSE);
					    END;
				       END;
				  END;
			     END;
		   | 2 :  IF (tsk^.next<>NIL)
			     THEN tsk:=tsk^.next;
			     ELSE IF (wq<>NIL) THEN tsk:=wq; f:=4;
				       ELSE IF (dq<>NIL) THEN tsk:=dq; f:=3;
					    ELSE RETURN (FALSE);
					    END;
				       END;
			     END;
		   | 4 :  IF (tsk^.nextdue<>NIL)
			     THEN tsk:=tsk^.nextdue;
			     ELSE IF (dq<>NIL) THEN tsk:=dq; f:=3;
					       ELSE RETURN (FALSE);
				  END;
			  END;
		   | 3 :  IF (tsk^.nextdue<>NIL)
			     THEN tsk:=tsk^.nextdue;
			     ELSE RETURN (FALSE);
			     END;
	      END;
	   IF (tsk^.NumProc=0)
	      THEN RETURN (NextTask(f,tsk));
	      ELSE RETURN(TRUE);
	      END;
       END NextTask;

 PROCEDURE Bankir_OK (): BOOLEAN;
(* ॠ ⬠  *)
       VAR
	   svob,fl,i    : CARDINAL;
	   Flag : BOOLEAN;
	   tsk     : Task;
     BEGIN
	   fl:=0;   i:=0;
	   tsk:=NIL;
	   WHILE (NextTask(fl,tsk))
	      DO tsk^.Finish:=TRUE; INC(i); END;
	   svob:=ResourceSet;
	   Flag:=TRUE;
	   WHILE (Flag) DO
                 Flag:=FALSE;
	         fl:=0;tsk:=NIL;
		 WHILE (NextTask(fl,tsk)) DO
		    IF ((tsk^.Finish) AND (tsk^.OstR<=svob))
			  THEN tsk^.Finish:=FALSE;
			       svob:=svob+tsk^.ActR;
			       Flag:=TRUE;
			  END;
		    END; (* nextt *)
	         END; (* while *)
	   RETURN (svob=ResourceLim);
       END Bankir_OK;

  PROCEDURE BankReady(t: Task; N: CARDINAL) : BOOLEAN;
  (* 䥩 ⬠  *)
	VAR Flag : BOOLEAN;
	    i : INTEGER;
      BEGIN ResourceSet:=ResourceSet-N;
	   GetInfo;
	   Window.Use(WinBank) ;
           IO.WrLn;
	   IO.WrStr ('>Quest.: process ');
	    WITH t^
	      DO ActR:=ActR+N;
		 OstR:=OstR-N;
	   IO.WrCard (NumProc,2);
	   IO.WrStr(' for');
	   IO.WrCard (N,2);
	   IO.WrStr(' res.-> ');
	         Flag:=NOT(Bankir_OK());
	         IF (Flag)
		    THEN ResourceSet:=ResourceSet+N;
		         ActR:=ActR-N;
			 OstR:=OstR+N;
                         IO.WrStr('No ');
                   ELSE  IO.WrStr('Yes');
		     END;
              END;
	   RETURN (NOT(Flag));
       END BankReady;

 (* Internal Use Only. End Part. *)
  (* ***  ⬠   *** *)

 (* *** 訫 ᠭ ॠ樨 ⬠  *** *)
 (* *** 楤 । ᮢ *** *)

 PROCEDURE InitResource ( VAR Num : CARDINAL);
 (*   ᮢ  न⭠ ਢ離 ᮢ *)
       VAR WD : Window.WinDef;
           i,lx,ly : CARDINAL;
           p,p1 : RESOURCE;
	   ie : CARDINAL;
     BEGIN
	   ie := GetFlags(); DI;
           Num := RANDOM(ResourceQuota-5)+5;
           ResourceLim := Num;
	   ResourceSet := Num;
	   RQ:=NIL;
           WD:=Window.WinDef (1,1,30,7,Window.LightGray,Window.Blue,FALSE,
                         TRUE,FALSE,TRUE,Window.SingleFrame,Window.White,
                         Window.Blue) ;
	   WinRes := Window.Open(WD) ;
           Window.Use(WinRes) ;
    Window.SetTitle(WinRes,' Resources Area ',Window.CenterUpperTitle) ;
    Window.Clear;
	   (*  ࠭ ⥩ ᮢ *)
           i:=1;
           FOR lx:=1 TO 21 BY 10 DO
               FOR ly:=1 TO 5 DO
               Window.GotoXY(lx,ly);
               IO.WrCard(i,2);
               IO.WrStr(':> ');
               IF i<=Num THEN IO.WrCard(0,2);
	   (*  ᯨ᪠ ResourceList *)
                ALLOCATE (p,SIZE(ResourceDescriptor));
               IF (i=1) THEN ResourceList:=p;
                        ELSE p1^.next:=p; END;
               p^.x:=lx+5;
               p^.y:=ly;
	       p1:=p;
		  ELSE IO.WrStr('  '); END;
               IF (lx<21) THEN  IO.WrStr(' | '); END;
               INC(i);
	   END; (* for *)
	   END;
	   p^.next:=NIL;
	   SetFlags(ie);
       END InitResource;

 PROCEDURE CLRResource (r : RESOURCE);
 (* 楤 ⪨   *)
     BEGIN Window.Use(WinRes);
	   Window.GotoXY(r^.x,r^.y);
	   Window.TextColor(CLRAtr);
	   IO.WrCard(0,2);
       END CLRResource;

 PROCEDURE USEResource (r : RESOURCE; t: CARDINAL);
 (* 楤    *)
     BEGIN
	   Window.Use(WinRes);
	   Window.TextColor(USEAtr);
           Window.GotoXY(r^.x,r^.y);
           IO.WrCard(t,2);
       END USEResource;

 PROCEDURE ACTResource (r : RESOURCE; t: CARDINAL);
 (* 楤 ⮡ࠦ 饭   *)
     BEGIN
	   Window.Use(WinRes);
           Window.TextColor(ACTAtr);
           Window.GotoXY(r^.x,r^.y);
           IO.WrCard(t,2);
       END ACTResource;

 PROCEDURE ResAlloc ( Num : CARDINAL; VAR pnt : RESOURCE);
 (* 頥 ᯨ᮪ Num ᢮ ᮢ *)
       VAR
	   p  : RESOURCE;
	   i  : CARDINAL;
     BEGIN
	   p:=ResourceList;
           pnt :=p;
           IF Num>1
              THEN
	          FOR i:=1 TO Num-1
	              DO p:=p^.next
		      END; (* for *)
             END;
	   ResourceList:=p^.next;
	   p^.next:=NIL;
       END ResAlloc;
 PROCEDURE Request ( Num : CARDINAL);
 (* 뤥  襭   ⠢ 
     RQ - ।   *)
       VAR
	   ie : CARDINAL;
	   mp : PROCESS;
	   t0 : Task;
     BEGIN
	   ie := GetFlags(); DI;
	   WITH cp^
		DO IF (Num<=ResourceSet) AND (BankReady(cp,Num))
		      THEN ResAlloc(Num,ListR);
		      ELSE NeedR:=Num;
			   t0:=cp;
			   cp:=cp^.next;
                           t0^.next:=NIL;
                           t0^.nextdue:=NIL;
			   QInsert(t0,RQ);
			   mp:=cp^.cor;
			   TRANSFER(mp,mp);
		      END;
		END;
	   SetFlags(ie);
       END Request;

 PROCEDURE Take ( VAR p : RESOURCE);
 (* 祭 㪠⥫  ᯨ᮪ 뤥 ᮢ *)
     BEGIN
	   p:=cp^.ListR;
	   cp^.NeedR := 0;
       END Take;

 PROCEDURE Free ( p : RESOURCE);
 (*   (᢮)  *)
       VAR
	   t0,t1 : Task;
	   Flag  : BOOLEAN;
     BEGIN
           CLRResource(p);
	   Window.Use(WinBank) ;
           IO.WrLn;
	   IO.WrStr ('>Return: process ');
	   IO.WrCard (cp^.NumProc,2);
	   IO.WrStr(' restore 1 res.');
	   p^.next:=ResourceList;
	   ResourceList:=p;
	   INC(ResourceSet);
           DEC(cp^.ActR);
	   IF (RQ<>NIL)
	      THEN Flag:=TRUE;
		   t1:=NIL; t0:=RQ;
		   WHILE (t0<>NIL) AND (Flag)
			 DO WITH t0^
			    DO
			       IF (NeedR<=ResourceSet) AND
                                  (BankReady(t0,NeedR))
				  THEN IF (t0=RQ)
					  THEN RQ:=t0^.next;
					  ELSE t1^.next:=next;
					  END; (* t0=rq *)
				       ResAlloc(NeedR,ListR);
                                       Flag:=FALSE;
                                       t0^.next:=NIL;
                                       t0^.nextdue:=wq;
                                       wq:=t0;
				  END; (* if flag *)
			    END; (* with *)
			 t1:=t0;
			 t0:=t0^.next;
			 END; (* while *)
	       END; (* if rq... *)
       END Free;
 (* *** 訫 ᠭ 楤 । ᮢ *** *)

 (* ***  *** *)

 PROCEDURE  StopProc;
 (*  訢襩  *)
       VAR
	   ie : CARDINAL;
	   mp : PROCESS;
	   t0 : Task;
     BEGIN
	   ie := GetFlags(); DI;
           t0:=cp;
	   cp:=cp^.next;
	   Window.Use(WinStat) ;
	   Window.GotoXY(1,2+GlNP);
	   IO.WrStr(' Process ');
	   IO.WrCard(t0^.NumProc,2);
	   IO.WrStr(' Parking... ');
           DEALLOCATE(t0,SIZE(TaskDescriptor));
	   mp:=cp^.cor;
           INC(GlNPZ); DEC(GlNP);
	   GetInfo;
           IF GlNP=0 THEN
	                  Window.Use(WinStat) ;
	                  Window.GotoXY(1,2);
 	                  IO.WrStr(' Game is over ... ');
                          StopFlag:=TRUE; StopScheduler;
           END;
	   TRANSFER(mp,mp);
           SetFlags(ie);
    END StopProc;

 (* ***  *** *)
 (* *** 楤 䥩 *** *)

 PROCEDURE Message (m:StrMes);
 (* 뢮 ⥬ ᮮ饭 *)
    VAR	   ie : CARDINAL;
           i : INTEGER;
     BEGIN
	   ie := GetFlags(); DI;
           Window.Use(WinBank);
           IO.WrLn;
           IO.WrStr (m);
           SetFlags(ie);
     END  Message;


 (* ⨪ *)
 PROCEDURE InitInfo;
(* *)
       VAR
	   ie : CARDINAL;
	   WD : Window.WinDef;
     BEGIN ie := GetFlags(); DI;
	   WD:=Window.WinDef (1,15,39,25,Window.LightGray,Window.Black,FALSE,
			 TRUE,FALSE,TRUE,Window.SingleFrame,Window.White,
			 Window.Black);
	   WinBank := Window.Open(WD);
	   Window.Use(WinBank);
    Window.SetTitle(WinBank,' Management Status ',Window.CenterUpperTitle) ;
    Window.Clear;
	   WD:=Window.WinDef (40,9,79,24,Window.LightGray,Window.Blue,FALSE,
			 TRUE,FALSE,TRUE,Window.SingleFrame,Window.White,
			 Window.Blue) ;
	   WinStat := Window.Open(WD) ;
	   Window.Use(WinStat) ;
    Window.SetTitle(WinStat,' Processes Status ',Window.CenterUpperTitle) ;
	   Window.Clear;
	   IO.WrStr(' Number    Max   Act   Reserv  Query ');
	   Window.GotoXY(1,12);
	   IO.WrStr(' Query Codes :'); IO.WrLn;
	   IO.WrStr('       1 - Active  2 - Wait Resources');IO.WrLn;
	   IO.WrStr('       3 - Wait Delay=Timer');
           Window.TextColor(Window.Yellow);
	   WD:=Window.WinDef (1,9,35,13,Window.Yellow,Window.Blue,FALSE,
			 TRUE,FALSE,TRUE,Window.DoubleFrame,Window.White,
			 Window.Blue) ;
	   WinLoc := Window.Open(WD) ;
	   Window.Use(WinLoc) ;
    Window.SetTitle(WinLoc,' Common  Statistic ',Window.CenterUpperTitle) ;
    Window.Clear;
	   IO.WrStr ('  Processes: Act=');
	   IO.WrCard(GlNP,2);
	   IO.WrStr ('  DeAct=');
	   IO.WrCard(GlNPZ,2);IO.WrLn;
	   IO.WrStr ('  Resources: Total=');
	   IO.WrCard(ResourceLim,2);
	   IO.WrStr ('  Clear=');
	   IO.WrCard(ResourceSet,2);IO.WrLn;
	   Window.TextColor(Window.LightGray);
	   IO.WrStr ('   Space - pause   Esc - exit');
           Window.TextColor(Window.LightGreen);
	   SetFlags(ie);
       END InitInfo;

 PROCEDURE GetInfo;
(* *)
       VAR
	   fl,i    : CARDINAL;
	   tsk     : Task;
	   ie : CARDINAL;
     BEGIN ie := GetFlags();  DI;
	   Window.Use(WinStat) ;
	   fl:=0;
	   tsk:=NIL;
	    Window.GotoXY(1,2);
	   WHILE (NextTask(fl,tsk))
	      DO WITH tsk^
		   DO IO.WrCard(NumProc,5);IO.WrCard(MaxR,8);
		      IO.WrCard(ActR,6);IO.WrCard(OstR,8);IO.WrCard(fl,7);
		      IO.WrLn;
		   END;
	   END;
	   IO.WrStr('');
	   Window.Use(WinLoc) ;
	    Window.GotoXY(18,1);
	   IO.WrCard(GlNP,2);
	    Window.GotoXY(28,1);
	   IO.WrCard(GlNPZ,2);
	    Window.GotoXY(20,2);
	   IO.WrCard(ResourceLim,2);
	    Window.GotoXY(30,2);
	   IO.WrCard(ResourceSet,2);
	   SetFlags(ie);
       END GetInfo;

 (* ***  *** *)



BEGIN
  dq := NIL;
  wq := NIL;
  GlNP:=0; GlNPZ:=0;
  NEW(cp);
  WITH cp^
    DO
       nextdue:=NIL;
       next := NIL;
       priority := 1;
       cor  := CurrentProcess();
       MaxR:=0;ActR:=0;OstR:=0;NeedR:=0;NumProc:=0;
		     ListR:=NIL;
    END;
  StartProcess( Idler, 512 , 0 ,0 ,0);
  SchedStack := NIL;
  Started  := FALSE;
  LockNest := 0;
  Terminate(CloseDown,Continue);
END Manager.



